#ifndef AdapterDSS_H
#define AdapterDSS_H

#include "DSS.h"
/*!
 * \file AdapterDSS.h
 * \brief Definition d'un adaptateur aux DSS
 * \author Tristan Roussillon
 *
 */


/*!
 * \class AdapterDSS
 * \brief Adaptateur aux DSS.
 * Cette classe, abstraite, modelise un adaptateur aux DSS
 * selon le modele de conception de l'adaptateur (adapter).
 * Comme les points d'appui superieurs (resp. inferieurs) 
 * sont toujours situes a droite (resp. a gauche) du segment,  
 * appeler les methodes getXf() et getXl() permet
 * d'acceder soit aux points d'appui superieurs, 
 * si on considere la droite du segment (AdapterDSS1)
 * soit aux points d'appui inferieurs,
 * si on considere la gauche du segment (AdapterDSS0)
 * \author Tristan Roussillon
 */
class AdapterDSS {
protected : 
	DSS *ps; /*!< pointeur vers un DSS*/ 
public : 
/*!
 * \brief Methode permettant 
 * d'ajouter un point M a l'avant d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si m union DSS est encore un DSS, 
 * FALSE sinon.
 * Si M est mal connecte au dernier point du segment,
 * une exception BadConnectivity est lancee.
 * \param  le point a ajouter au DSS
 */
	bool push_front(const Point<int>& );
/*!
 * \brief Methode permettant 
 * d'ajouter un point M a l'arriere d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si m union DSS est encore un DSS, 
 * FALSE sinon.
 * Si M est mal connecte au premier point du segment,
 * une exception BadConnectivity est lancee.
 * \param  le point a ajouter au DSS
 */
	bool push_back(const Point<int>& );
/*!
 * \brief Methode permettant 
 * de supprimer un point M a l'avant d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si la suppression est possible
 * (le segment contient au moins 2 points), 
 * FALSE sinon.
 * Si M n'est pas le dernier point du segment,
 * une exception BadConnectivity est lancee.
 * \param un point
 */
	bool pop_front(const Point<int>& );
/*!
 * \brief Methode permettant 
 * de supprimer un point M a l'arriere d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si la suppression est possible
 * (le segment contient au moins 2 points), 
 * FALSE sinon.
 * Si M n'est pas le premier point du segment,
 * une exception BadConnectivity est lancee.
 * \param un point
 */
	bool pop_back(const Point<int>& );

/*!
 * \brief Methode 
 * permettant d'acceder a 
 * la composante x de la 
 * normale du segment
 * 
 */
	int getA() const;
/*!
 * \brief Methode 
 * permettant d'acceder a 
 * la composante y de la 
 * normale du segment
 * 
 */
	int getB() const;
/*!
 * \brief Methode permettant d'acceder a l'intercept
 * situe a droite - c1 - du segment
 * 
 */
	int getC1() const;
/*!
 * \brief Methode permettant d'acceder a l'intercept
 * situe a gauche - c2 - du segment
 * 
 */
	int getC2() const;
/*!
 * \brief Methode virtuelle pure, 
 * permettant d'acceder a l'intercept
 * situe a droite - c1 - (AdapterDSS1) 
 * ou a gauche - c2 - (AdapterDSS0)
 * du segment
 * 
 */
	virtual int getC() const = 0;

/*!
 * \brief Methode permettant d'acceder au reste
 * du point M donne en parametre :
 * M.x*a - M.y*b - c 
 */
	int r(const Point<int>& ) const;

/*!
 * \brief Methode virtuelle pure, 
 * permettant d'acceder premier point d'appui
 * superieur (AdapterDSS1) ou inferieur (AdapterDSS0)
 * 
 */
	virtual Point<int> getXf() const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * permettant d'acceder dernier point d'appui
 * superieur (AdapterDSS1) ou inferieur (AdapterDSS0)
 * 
 */
	virtual Point<int> getXl() const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * permettant d'acceder premier point d'appui
 * inf (AdapterDSS1) ou sup (AdapterDSS0)
 * 
 */
	virtual Point<int> getYf() const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * permettant d'acceder dernier point d'appui
 * inf (AdapterDSS1) ou sup (AdapterDSS0)
 * 
 */
	virtual Point<int> getYl() const = 0;

/*!
 * \brief premier point
 */
	Point<int> getFirst() const;
/*!
 * \brief dernier point
 */
	Point<int> getLast() const;

/*!
 * \brief predicat indicant si un point 
 * appartient au segment ou non
 *
 */
 	bool isInside(const Point<int>& );

	virtual ~AdapterDSS() {}
};

#endif

