#ifndef AdapterIterator_H
#define AdapterIterator_H
/*!
 * \file AdapterIterator.h
 * \brief Definition d'une classe modelisant 
 * un adaptateur a un iterateur sur un conteneur
 * (selon le design pattern adapter).
 *
 * \author Tristan Roussillon
 *
 */


/*!
 * \class AdapterIterator
 * \brief Classe generique et abstraite modelisant 
 * un adaptateur a un iterateur sur un conteneur
 * (selon le design pattern adapter).
 *
 * \author Tristan Roussillon
 * 
*/
//T type de l'element dans le conteneur
//Ti type de l'iterateur
template <class T, class Ti> class AdapterIterator {
protected : 
	Ti *i; /*!< iterateur sur un conteneur */
public : 
/*!
 * \brief Methode virtuelle permettant
 * d'acceder a l'element suivant du conteneur
 */
	virtual void operator++(int) = 0;
/*!
 * \brief Methode virtuelle permettant
 * d'acceder a l'element precedent du conteneur
 */
	virtual void operator--(int) = 0;

/*!
 * \brief Acceder a l'iterateur
 */
	Ti getI() const;
/*!
 * \brief Acceder a la variable vers laquelle pointe l'iterateur
 */
	T operator*();
/*!
 * \brief Operateur d'affectation
 * \param iterateur
 */
	AdapterIterator<T,Ti>& operator=(Ti); 
/*!
 * \brief Operateur d'affectation
 * \param Adaptateur a un iterateur
 */
	AdapterIterator<T,Ti>& operator=(const AdapterIterator<T,Ti>&);
/*!
 * \brief Operateur d'egalite
 * \param iterateur
 */
	bool operator==(const Ti&) const; 
/*!
 * \brief Operateur d'egalite
 * \param Adaptateur a un iterateur
 */
	bool operator==(const AdapterIterator<T,Ti>&) const; 
/*!
 * \brief Operateur de difference
 * \param iterateur
 */
	bool operator!=(const Ti&) const; 
/*!
 * \brief Operateur d'egalite
 * \param Adaptateur a un iterateur
 */
	bool operator!=(const AdapterIterator<T,Ti>&) const; 
};

template <class T, class Ti> Ti AdapterIterator<T,Ti>::getI() const {
	return (*this->i);
}

template <class T, class Ti> T AdapterIterator<T,Ti>::operator*() {
	return *(*this->i);
}

template <class T, class Ti> 
AdapterIterator<T,Ti>& AdapterIterator<T,Ti>::operator=(Ti it) {
	*(this->i) = it;
	return *this;
}

template <class T, class Ti> 
AdapterIterator<T,Ti>& AdapterIterator<T,Ti>::operator=(const AdapterIterator<T,Ti>& a) {
	Ti it = a.getI();
	*(this->i) = it;
	return *this;
}

template <class T, class Ti> 
bool AdapterIterator<T,Ti>::operator==(const Ti& it) const {
	return (*(this->i) == it);
} 

template <class T, class Ti> 
bool AdapterIterator<T,Ti>::operator==(const AdapterIterator<T,Ti>& a) const {
	return (*(this->i) == (a.getI()));
} 

template <class T, class Ti> 
bool AdapterIterator<T,Ti>::operator!=(const Ti& it) const {
	return (*(this->i) != it);
} 

template <class T, class Ti> 
bool AdapterIterator<T,Ti>::operator!=(const AdapterIterator<T,Ti>& a) const {
	return (*(this->i) != (a.getI()));
} 

//virtual ~AdapterIterator() {}
#endif
