#ifndef DSS_H
#define DSS_H

#include "BadConnectivity.h"
#include "PushPopSeg.h"
#include <sstream>
#include <fstream>
/*!
 * \file DSS.h
 * \brief Definition d'une classe modelisant un segment de droite
 * selon sa caracterisation arithmetique (reveilles91, debled95)
 * \author Tristan Roussillon
 *
 */


/*!
 * \class DSS
 * \brief segment de droite arithmetique
 * \author Tristan Roussillon
 * cette classe, abstraite, modelise un segment de droite arithmetique.
 */
class DSS : public PushPopSeg {

/*!
 * \brief operateur de sortie
 */
	friend std::ostream& operator<<(std::ostream&, const DSS&); 

public : 
/*!
 * \brief Constructeur
 * \param 2 points  
 */
	DSS(const Point<int>&, const Point<int>&); 
/*!
 * \brief Constructeur de copie
 * \param un DSS
 */
	DSS(const DSS&);
/*!
 * \brief Destructeur
 */
	virtual ~DSS() {} 
/*!
 * \brief Operateur d'affectation
 * \param un DSS
 */
	DSS& operator=(const DSS&); 


/*!
 * \brief Methode permettant 
 * d'ajouter un point M a l'avant d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si m union DSS est encore un DSS, 
 * FALSE sinon.
 * Si M est mal connecte au dernier point du segment,
 * une exception BadConnectivity est lancee.
 * \param  le point a ajouter au DSS
 */
	bool push_front(const Point<int>& );
/*!
 * \brief Methode permettant 
 * d'ajouter un point M a l'arriere d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si m union DSS est encore un DSS, 
 * FALSE sinon.
 * Si M est mal connecte au premier point du segment,
 * une exception BadConnectivity est lancee.
 * \param  le point a ajouter au DSS
 */
	bool push_back(const Point<int>& );
/*!
 * \brief Methode permettant 
 * de supprimer un point M a l'avant d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si la suppression est possible
 * (le segment contient au moins 2 points), 
 * FALSE sinon.
 * Si M n'est pas le dernier point du segment,
 * une exception BadConnectivity est lancee.
 * \param un point
 */
	bool pop_front(const Point<int>& );
/*!
 * \brief Methode permettant 
 * de supprimer un point M a l'arriere d'un DSS
 * (selon le sens de parcours de la courbe)
 * et renvoie TRUE si la suppression est possible
 * (le segment contient au moins 2 points), 
 * FALSE sinon.
 * Si M n'est pas le premier point du segment,
 * une exception BadConnectivity est lancee.
 * \param un point
 */
	bool pop_back(const Point<int>& );

/*!
 * \brief predicat indicant si un point 
 * appartient au segment ou non
 *
 */
 	bool isInside(const Point<int>& );

/*!
 * \brief Accede a l'attribut a,
 * composante de la normale du segment. 
 */
	int getA();
/*!
 * \brief Accede a l'attribut b,
 * composante de la normale du segment. 
 */
	int getB();
/*!
 * \brief Accede a l'attribut c1,
 * multiplie par max(a,b), il est l'ordonnee a l'origne 
 * de la premiere droite d'appui. 
 */
	int getC1();
/*!
 * \brief Accede a l'attribut c2,
 * multiplie par max(a,b), il est l'ordonnee a l'origne 
 * de la seconde droite d'appui. 
 */
	int getC2();
/*!
 * \brief Accede a l'attribut c1 (resp. c2),
 * si le bouleen en entree est vrai (resp. faux). 
 * \param un bouleen
 */
	int getC(bool);
/*!
 * \brief Accede a l'attribut c1,
 * multiplie par max(a,b), il est l'ordonnee a l'origne 
 * de la premiere droite d'appui. 
 */
	int getMu();
/*!
 * \brief Donne l'epaisseur du segment,
 */
	virtual int getOmega() const = 0;
/*!
 * \brief Donne la pente du segment. 
 */
	double getSlope();
/*!
 * \brief Donne l'ordonnee a l'origine du segment. 
 */
	double getIntercept();
/*!
 * \brief Methode permettant d'acceder au reste
 * du point M donne en parametre :
 * M.x*a - M.y*b - c1 
 */
	int r(const Point<int>& );
/*!
 * \brief Methode 
 * retournant le premier point 
 * d'appui superieur du segment 
 */
	Point<int> getUf() const;
/*!
 * \brief Methode 
 * retournant le dernier point 
 * d'appui superieur du segment 
 */
	Point<int> getUl() const;
/*!
 * \brief Methode 
 * retournant le premier point 
 * d'appui inferieur du segment 
 */
	Point<int> getLf() const;
/*!
 * \brief Methode  
 * retournant le dernier point 
 * d'appui inferieur du segment 
 */
	Point<int> getLl() const;



/*!
 * \brief Methode 
 * retournant la representation 
 * chaine de caractere du segment
 *
 */
	std::string toString() const;


protected:

	//a, b, c1, c2 sont les caracteristiques de la droite discrete
	int a; /*!< composante de la normale du segment*/
	int b; /*!< composante de la normale du segment*/
	int c1; /*!< borne inferieure */
	int c2; /*!< borne superieur */
	//Uf, Ul, Lf, Ll, les points d'appui sup et inf (premier f et dernier l)
	Point<int> Uf; /*!< premier point d'appui superieur du segment */
	Point<int> Ul; /*!< dernier point d'appui superieur du segment */
	Point<int> Lf; /*!< premier point d'appui inferieur du segment */
	Point<int> Ll; /*!< dernier point d'appui inferieur du segment */

/*!
 * \brief Methode virtuelle pure, 
 * retournant le vecteur deplacement
 * permettant de trouver le point de Bezout
 * (point n'appartenant pas au segment le plus proche du segment)
 * a partir du point d'appui superieur
 * ou de trouver le point d'appui inferieur
 * a partir du point de Bezout. 
 *
 */
	virtual Vecteur<int> shift() const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * retournant le point suivant le
 * premier point du segment. 
 * \param pfirst : premier point du segment
 * \param plast : dernier point du segment
 *
 */
	virtual Point<int> nextPt(const Point<int>& pfirst, const Point<int>& plast) const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * retournant TRUE si P et Q sont *-voisins, 
 * FALSE sinon.
 * \param P : un point
 * \param Q : un point 
 *
 */
	virtual bool areNeighbors(const Point<int>& P, const Point<int>& Q) const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * retournant TRUE si N est strictement *-voisin de plast, 
 * FALSE sinon.
 * \param pfirst : premier point du segment
 * \param plast : dernier point du segment
 * \param N : le point ajoute
 *
 */
	virtual bool isBadConnected(const Point<int>& pfirst, const Point<int>& plast, const Point<int>& N) const = 0;

/*!
 * \brief Methode permettant 
 * d'ajouter un point M a un DSS
 * et renvoie TRUE si m union DSS est encore un DSS, 
 * FALSE sinon.
 */
	bool push(const Point<int>& M, Point<int>& rend, Point<int>& end, 
			Point<int>& pUf, Point<int>& pUl, Point<int>& pLf, Point<int>& pLl, 
			int pa, int pb, int pc1, int pc2);
/*!
 * \brief Methode permettant 
 * de supprimer un point M a un DSS
 * et renvoie TRUE si la suppression est possible
 * (le segment contient au moins 2 points), 
 * FALSE sinon.
 */
	bool pop(const Point<int>& M, Point<int>& pfirst, Point<int>& plast,
			Point<int>& pUf, Point<int>& pUl, Point<int>& pLf, Point<int>& pLl, int k);

/*!
 * \brief Methode permettant 
 * de mettre a jour les caracteristiques d'un DSS
 * lors de la suppression d'un point
 */
	void update(const Point<int>& P0, const Point<int>& Pn, 
			Point<int>& U1, Point<int>& U2, Point<int>& L1, Point<int>& L2,
			const Vecteur<int>& v, int k);


};

     
#endif

