#ifndef PartialConvexHull_H
#define PartialConvexHull_H

#include <deque> //file a double extremite
#include <vector>

#include "Point.h"
#include "AdapterContainer.h" //adaptateur pour deque
#include "AdapterContainerFront.h" //adaptateur pour deque
#include "AdapterContainerBack.h" //adaptateur pour deque
#include "Predicat.h" //predicat d'orientation

/*!
 * \file PartialConvexHull.h
 * \brief Definition d'une classe permettant 
 * de calculer la ligne polygonale reliant
 * le premier et dernier point d'une courbe
 * telle qu'elle est toujours d'un cote
 * de cette courbe (selon le predicat)
 * et qu'elle est la plus courte possible
 * \author Tristan Roussillon
 *
 */


/*!
 * \class PartialConvexHull
 * \brief Classe generique modelisant 
 * l'enveloppe convexe partielle 
 * d'un ensemble de points
 * \author Tristan Roussillon
 * 
*/
//T type de coordonnees des points
//Tc type du conteneur (qui doit contenir des Point<T>)
template <class T, class Tc> 
class PartialConvexHull {

protected :
	//membres donnees
	Tc dCH; /*!< conteneur stockant l'enveloppe convexe */
	Predicat<T>* pred; /*!< predicat d'orientation */
	int nb; /*!< nb de points ajoutes */
	bool added2Front; /*!< bouleen mis a TRUE si le dernier point a ete ajoute a l'avant
	et mis a FALSE si le dernier point a ete ajoute a l'arriere */


/*!
 * \brief Ajouter un point  l'enveloppe convexe courante
 * \param n : un point
 * \param aCH : pointeur vers un adaptateur du conteneur 
 * stockant les points de l'enveloppe  
 */
	virtual void addPoint(const Point<T>& n); 

public :
/*!
 * \brief Constructeur par defaut
 */
	PartialConvexHull();
/*!
 * \brief Constructeur
 * \param p : un point
 * \param predicat : predicat d'orientation  
 */
	PartialConvexHull(const Point<T>&, Predicat<T>* predicat);
/*!
 * \brief Constructeur
 * \param p : un point
 * \param q : un point
 * \param predicat : predicat d'orientation  
 */
	PartialConvexHull(const Point<T>&, const Point<T>&, Predicat<T>* predicat);
/*!
 * \brief Destructeur
 */
	virtual ~PartialConvexHull();

/*!
 * \brief Ajouter un point a l'avant de l'enveloppe 
 * \param n : un point a ajouter
 */
	void push_front(const Point<T>& n); 
/*!
 * \brief Ajouter un point a l'arriere de l'enveloppe 
 * \param n : un point a ajouter
 */
	void push_back(const Point<T>& n); 
/*!
 * \brief Acceder a l'enveloppe 
 */
	std::vector<Point<T> > getCH() const;	
/*!
 * \brief Acceder au nombre de sommet de l'enveloppe
 */
	virtual int getSize() const;
/*!
 * \brief Acceder au nombre de points traites
 */
	int getNb() const;
/*!
 * \brief Acceder a l'iterateur avant 
 * du conteneur stockant l'enveloppe convexe
 */
	typename Tc::iterator iFront();
/*!
 * \brief Acceder a l'iterateur arriere 
 * du conteneur stockant l'enveloppe convexe
 */
	typename Tc::iterator iBack();
};

template <class T, class Tc> 
PartialConvexHull<T,Tc>::~PartialConvexHull() {
	delete this->pred;
}

template <class T, class Tc> 
PartialConvexHull<T,Tc>::PartialConvexHull() {
	this->nb = 0;
	this->pred = new Predicat<T>();
} 

template <class T, class Tc> 
PartialConvexHull<T,Tc>::PartialConvexHull(
	const Point<T>& m, 
	Predicat<T>* predicat) {

	this->nb = 1;
	this->pred = predicat;
	this->dCH.push_front(m);
} 

template <class T, class Tc> 
PartialConvexHull<T,Tc>::PartialConvexHull(
	const Point<T>& m, 
	const Point<T>& n, 
	Predicat<T>* predicat) {

	this->nb = 2;
	this->pred = predicat;
	this->dCH.push_front(m);
	this->dCH.push_front(n);

}

//ajout d'un point a l'avant de l'enveloppe convexe
template <class T, class Tc> 
void PartialConvexHull<T,Tc>::push_front(const Point<T>& n) {
	this->added2Front = true;
	this->addPoint(n);
}

//ajout d'un point a l'arriere de l'enveloppe convexe
template <class T, class Tc> 
void PartialConvexHull<T,Tc>::push_back(const Point<T>& n) {
	this->added2Front = false;
	this->pred->ReverseSign();
	this->addPoint(n);
	this->pred->ReverseSign();
}

//ajout d'un point  l'enveloppe convexe
template <class T, class Tc> 
void PartialConvexHull<T,Tc>::addPoint(const Point<T>& n) {

	AdapterContainer<Point<T>,Tc> *aCH;
	if (this->added2Front) {	
		aCH = new AdapterContainerFront<Point<T>,Tc>(&this->dCH);
	} else {
		aCH = new AdapterContainerBack<Point<T>,Tc>(&this->dCH);
	}

	#ifdef DEBUG_PRED
	std::cout  << "AJOUT du point n " << n << std::endl;
	#endif

	if (this->nb > 1) {
	
		Point<T> p, q;
		q = aCH->end();
		aCH->pop();
		p = aCH->end();
		aCH->push(q);

		while ((this->dCH.size() > 1) && 
			(this->pred->isNoGood(p,q,n))) {
		
			#ifdef DEBUG_PRED
			std::cout << "\t SUPPR : " << aCH->end() << std::endl;
			#endif 

			aCH->pop();

			if (this->dCH.size() > 1) {
				q = aCH->end();
				aCH->pop();
				p = aCH->end();
				aCH->push(q);
			}
		}	
	}
	this->nb++;
	aCH->push(n);
}

template <class T, class Tc> 
std::vector<Point<T> > PartialConvexHull<T,Tc>::getCH() const {

	std::vector<Point<T> > res;
	res.assign( this->dCH.rbegin(),  this->dCH.rend() );
	return res;
}

template <class T, class Tc> 
int PartialConvexHull<T,Tc>::getSize() const {
	return this->dCH.size();
}

template <class T, class Tc> 
int PartialConvexHull<T,Tc>::getNb() const {
	return this->nb;
}

template <class T, class Tc> 
typename Tc::iterator PartialConvexHull<T,Tc>::iFront() {
	return this->dCH.begin();
}

template <class T, class Tc> 
typename Tc::iterator PartialConvexHull<T,Tc>::iBack() {
	typename Tc::iterator it = this->dCH.end();
	it--;
	return it;
}

   #endif

