#ifndef POINT_H
   #define POINT_H

#include "Vecteur.h"
/*!
 * \file Point.h
 * \brief  dfinition et implementation de la classe Point
 * \author Tristan Roussillon
 *
 */

/*!
 * \class Point
 * \brief Classe generique pour traiter des points 2D
 * \author Tristan Roussillon
 *
 */
template <class T> class Point {

public:
	T x;/*!< abscisse*/
	T y;/*!< ordonnee */

/*!
 * \brief constructeur par defaut
 */
	Point();
/*!
 * \brief constructeur
 */ 
	Point(T c1, T c2);
/*!
 * \brief constructeur de copie
 */ 
	Point(const Point<T>&); /*!< constructeur de copie*/
/*!
 * \brief destructeur
 */ 
	virtual ~Point() {}
/*!
 * \brief operateur d'affectation
 */ 
	Point<T>& operator=(const Point<T>&);

/*!
 * \brief operateur d'egalite
 */ 
	bool operator==(const Point<T>& p) const; 
/*!
 * \brief operateur de difference
 */ 
	bool operator!=(const Point<T>& p) const; 
/*!
 * \brief addition d'un vecteur  un point 
 */ 
	Point<T> operator+(const Vecteur<T>& pq) const; 
/*!
 * \brief  soustraction d'un vecteur a un point 
 */ 
	Point<T> operator-(const Vecteur<T>& qp) const; 
/*!
 * \brief  soustraction entre deux points
 */ 
	Vecteur<T> operator-(const Point<T>& q) const; 
/*!
 * \brief operateur d'ecriture
 */ 
	std::ostream& operator<<(const Point<T>& p); 
/*!
 * \brief distance de Manhattan
 */ 
	T d1(const Point<T>& p) const; 
/*!
 * \brief distance euclidienne
 */ 
	double d2(const Point<T>& p) const; 
/*!
 * \brief distance euclidienne au carre
 */ 
	T d22(const Point<T>& p) const; 
/*!
 * \brief distance de Chebytchev
 */ 
	T dmax(const Point<T>& p) const; 
/*!
 * \brief operateur de conversion en entier
 */
Point<int> castinint();
/*!
 * \brief operateur de conversion en long
 */
Point<long> castinlong();
/*!
 * \brief operateur de conversion en double
 */
Point<double> castindouble();
/*!
 * \brief operateur de conversion en float
 */
Point<float> castinfloat();

};

//implementation du constructeur par defaut
template <class T> Point<T>::Point() {}
//implementation du constructeur par sa liste d'initialisation
template <class T> Point<T>::Point(T c1, T c2) : x(c1), y(c2) {}
//constructeur de copie
template <class T> Point<T>::Point(const Point<T>& p) : x(p.x), y(p.y) {}

//oprateur d'affectation
template <class T> Point<T>& Point<T>::operator=(const Point<T>& p) {
          x = p.x;
          y = p.y;
          return *this;
}

//surcharge de l'operateur ==
//deux points sont gaux ssi leurs abscisses sont gales et leurs ordonnes sont gales
template <class T> bool Point<T>::operator==(const Point<T>& p) const {
	return ((p.x == this->x) && (p.y == this->y));
}
template <class T> bool Point<T>::operator!=(const Point<T>& p) const {
	return ((p.x != this->x) || (p.y != this->y));
}

//surcharge de l'operateur plus
//le point P plus le vecteur PQ egale le point Q
template <class T> Point<T> Point<T>::operator+(const Vecteur<T>& pq) const {
	Point<T> q(pq.x + this->x, pq.y + this->y);
	return q;
}

//surcharge de l'operateur moins
//le point P moins le vecteur QP egale le point Q
template <class T> Point<T> Point<T>::operator-(const Vecteur<T>& qp) const {
	Point<T> q(this->x - qp.x, this->y - qp.y);
	return q;
}
//le point Q moins le point P egale le vecteur PQ (de P vers Q)
template <class T> Vecteur<T> Point<T>::operator-(const Point<T>& p) const {
	Vecteur<T> v(this->x - p.x, this->y - p.y);
	return v;
}

//surcharge de l'operateur d eriture
template <class T> std::ostream& operator<<(std::ostream& os, const Point<T>& p) {
	os << "(" << p.x << ", " << p.y << ") ";
	return os;
}

//calcul de distance entre deux points
template <class T> T Point<T>::d1(const Point<T>& p) const {
	return (abs(p.x - this->x) + abs(p.y - this->y));
}
template <class T> double Point<T>::d2(const Point<T>& p) const {
	return sqrt(((p.x - this->x)*(p.x - this->x)) + ((p.y - this->y)*(p.y - this->y)));
}
template <class T> T Point<T>::d22(const Point<T>& p) const {
	return ((p.x - this->x)*(p.x - this->x)) + ((p.y - this->y)*(p.y - this->y));
}
template <class T> T Point<T>::dmax(const Point<T>& p) const {
	T dx = abs(p.x - this->x);
	T dy = abs(p.y - this->y);
	return dx>=dy?dx:dy;
}

template <class T>
Point<int> Point<T>::castinint() {
   return Point<int>((int)this->x,(int)this->y);
}

template <class T>
Point<long> Point<T>::castinlong() {
   return Point<long>((long)this->x,(long)this->y);
}

template <class T>
Point<double> Point<T>::castindouble() {
   return Point<double>((double)this->x,(double)this->y);
}
template <class T>
Point<float> Point<T>::castinfloat() {
   return Point<float>((float)this->x,(float)this->y);
}
   #endif


