#ifndef UIFIG_H
   #define UIFIG_H

/*!
 * \file UIfig.h
 * \brief Definition d'une classe (UIfig)
 * permettant de dessiner 
 * des ensembles de points ou lignes 
 * dans un fichier SVG
 * \author Tristan Roussillon
 *
 */

#include "Point.h"
#include <vector>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <string>

/*!
 * \class UIfig
 * \brief Classe permettant de 
 * dessiner des ensembles points ou lignes 
 * dans un fichier SVG
 * \author Tristan Roussillon
 */
class UIfig{

public:

	std::fstream fic; /*!< flux */

/*!
 * \brief Constructeur
 * \param nomfic: nom du fichier
 * \param v: points du contour
 */
	UIfig(const std::string nomfic, const std::vector<Point<int> >& v);

/*!
 * \brief Destructeur
 */

	~UIfig();


	//fonctions de dessin
/*!
 * \brief Methode permettant 
 * de dessiner des lignes.
 */
	template <class T>
	void addPolyline(
			const std::vector<Point<T> >& v, 
			const std::string& name, 
			const std::string& couleur);


/*!
 * \brief Methode permettant 
 * de dessiner des pixels (sous
 * la forme de carré unité).
 */
	template <class T>
	void addPixels(
			const std::vector<Point<T> >& v, 
			const std::string& name, 
			const std::string& couleur);

/*!
 * \brief Methode permettant 
 * de dessiner des marques (sous
 * la forme de petits disques)
 */
	template <class T>
	void addCircularMarks(
			const std::vector<Point<T> >& v, 
			const std::string& name, 
			const std::string& couleur);

/*!
 * \brief Methode permettant 
 * de dessiner un segment de droite
 * sous la forme d'une boite englobante
 * definie par des points d'appui
 */
	void addSegFromLeaningPoints(
			const Point<int>& first,
			const Point<int>& last,  
			const Point<int>& Uf, 
			const Point<int>& Ul, 
			const Point<int>& Lf, 
			const Point<int>& Ll, 
			const std::string& couleur);

};

template <class T>
void UIfig::addPolyline(
		const std::vector<Point<T> >& v, 
		const std::string& name, 
		const std::string& couleur) {

	this->fic << "<g id='" << name << "'>" << std::endl;

	this->fic << "<polyline fill='none' stroke='" << couleur << "' stroke-width='0.1' ";
	this->fic << "points='";
	//liste des points 
	for (int i = 0; i < v.size(); i++) {
		Point<T> p = v.at(i);
		this->fic << p.x << "," << p.y << " ";
	}
	this->fic << "' />" << std::endl;

	this->fic << "</g>" << std::endl;
}

template <class T>
void UIfig::addPixels(
		const std::vector<Point<T> >& v, 
		const std::string& name, 
		const std::string& couleur) {

	this->fic << "<g id='" << name << "'>" << std::endl;

	//liste des points 
	for (int i = 0; i < v.size(); i++) {
		Point<T> p = v.at(i);
		this->fic << "<rect fill='none' stroke='" << couleur << "' stroke-width='0.1' ";
		this->fic << "x='" << p.x-0.5 << "' y='" << p.y-0.5 << "' width='1' height='1' />";
	}

	this->fic << "</g>" << std::endl;
}

template <class T>
void UIfig::addCircularMarks(
		const std::vector<Point<T> >& v, 
		const std::string& name, 
		const std::string& couleur) {

	this->fic << "<g id='" << name << "'>" << std::endl;

	//liste des points 
	for (int i = 0; i < v.size(); i++) {
		Point<T> p = v.at(i);
		this->fic << "<circle ";
		this->fic << "cx='" << p.x << "' ";
		this->fic << "cy='" << p.y << "' ";
		this->fic << "r='0.2' ";
		this->fic << "fill='" << couleur << "' ";
		this->fic << "stroke='" << couleur << "' ";
		this->fic << "stroke-width='0.1'/>" << std::endl;
	}

	this->fic << "</g>" << std::endl;
}
  #endif


