#ifndef VECTEUR_H
   #define VECTEUR_H

#include <iostream>
#include <cmath> 
#include <cstdlib>
/*!
 * \file Vecteur.h
 * \brief  definition et implementation de la classe Vecteur
 * \author Tristan Roussillon
 *
 */

/*!
 * \class Vecteur
 * \brief  classe pour les donnees et traitements d'un vecteur 2D
 * \author Tristan Roussillon
 * Cette classe est une classe generique pour traiter des vecteurs 2D
 *
 */

template <class T> class Vecteur{

public:
	T x; /*!< composante x */
	T y; /*!< composante y */

	Vecteur(); /*!<  constructeur par defaut */
	Vecteur(T c1, T c2); /*!< constructeur */
	Vecteur(const Vecteur<T>&); /*!< constructeur de copie */
	~Vecteur(); /*!< destructeur */
	Vecteur<T>& operator=(const Vecteur<T>&); /*!< operateur d'affectation */

	bool operator==(const Vecteur<T>& p) const; /*!< operateur d'egalite */

	Vecteur<T> operator+(const Vecteur<T>& v) const; /*!< addition de deux vecteurs */
	Vecteur<T> operator-(const Vecteur<T>& v) const; /*!<  soustraction entre deux vecteurs */
	Vecteur<T> operator-() const; /*!< vecteur oppose */
	T operator*(const Vecteur<T>& v) const; /*!< multiplication entre deux vecteurs : produit scalaire*/
	Vecteur<T> operator*(const T& k) const; /*!< multiplication par un scalaire*/

	double norme() const; /*!< norme du vecteur */
	T norme2() const; /*!< norme du vecteur au carre */
	double pente() const; /*!< pente du vecteur */

	std::ostream& operator<<(const Vecteur<T>& v); /*!<  operateur d'ecriture*/ 

};

//implmentation du constructeur par dfaut
template <class T> Vecteur<T>::Vecteur() {}
//implmentation du constructeur par sa liste d'initialisation
template <class T> Vecteur<T>::Vecteur(T c1, T c2) : x(c1), y(c2) {}
//constructeur de copie
template <class T> Vecteur<T>::Vecteur(const Vecteur<T>& p) : x(p.x), y(p.y) {}
//destructeur
template <class T> Vecteur<T>::~Vecteur() {}

//oprateur d'affectation
template <class T> Vecteur<T>& Vecteur<T>::operator=(const Vecteur<T>& p) {
          x = p.x;
          y = p.y;
          return *this;
}

//surcharge de l'operateur ==
//deux vecteurs sont gaux ssi leurs composantes sont gales deux  deux
template <class T> bool Vecteur<T>::operator==(const Vecteur<T>& p) const {
	return ((p.x == this->x) && (p.y == this->y));
}
//surcharge de l'operateur plus
template <class T> Vecteur<T> Vecteur<T>::operator+(const Vecteur<T>& v) const {
	Vecteur<T> somme(v.x + this->x, v.y + this->y);
	return somme;
}

//surcharge de l'operateur moins
template <class T> Vecteur<T> Vecteur<T>::operator-(const Vecteur<T>& v) const {
	Vecteur<T> difference(this->x - v.x, this->y - v.y);
	return difference;
}

//surcharge de l'operateur moins
template <class T> Vecteur<T> Vecteur<T>::operator-() const {
	Vecteur<T> oppose(-this->x, -this->y);
	return oppose;
}

//surcharge de l'operateur multiplie (produit scalaire)
template <class T> T Vecteur<T>::operator*(const Vecteur<T>& v) const {
	return ((this->x * v.x) + (this->y * v.y));
}

//surcharge de l'operateur multiplication par un scalaire
template <class T> Vecteur<T> Vecteur<T>::operator*(const T& k) const {
	Vecteur<T> scaledv(this->x*k, this->y*k);
	return scaledv;
}

//norme (longueur) d'un vecteur
template <class T> double Vecteur<T>::norme() const {
	return sqrt(this->x*this->x + this->y*this->y);
}
//norme (longueur) d'un vecteur
template <class T> T Vecteur<T>::norme2() const {
	return (this->x*this->x + this->y*this->y);
}
 
//pente d'un vecteur
template <class T> double Vecteur<T>::pente() const {
	return (double) this->y/this->x;
}

//surcharge de l'operateur d'ecriture
template <class T> std::ostream& operator<<(std::ostream& os, const Vecteur<T>& v) {
	os << "(" << v.x << "," << v.y << ") ";
	return os;
}

   #endif
